"""
web2ldap plugin classes for attributes defined for msPerson

$Id: msperson.py,v 1.4 2005/06/21 06:58:14 michael Exp $
"""

import pyweblib.forms

from w2lapp.core import utf2display

from w2lapp.schema.syntaxes import LDAPSyntax,Boolean,Integer,IA5String,SelectList,syntax_registry


class Gender(SelectList):
  oid = 'Gender-oid'
  desc = 'Representation of human sex (see ISO 5218)'

  attr_value_dict = {
    '':'',
    '0':'Not known',
    '1':'Male',
    '2':'Female',
    '9':'Not specified',
  }

syntax_registry.registerAttrType(
  Gender.oid,[
    '1.3.6.1.4.1.5427.1.389.4.7', # gender (defined for msPerson)
  ]
)


class LabeledIBAN(IA5String):
  """
  More information:
  http://de.wikipedia.org/wiki/International_Bank_Account_Number
  http://www.pruefziffernberechnung.de/I/IBAN.shtml
  """
  oid = 'LabeledIBAN-oid'
  desc = 'International bank account number (IBAN) syntax (see ISO 13616:1997)'

syntax_registry.registerAttrType(
  Gender.oid,[
    '1.3.6.1.4.1.5427.1.389.4.13', # labeledIBAN
  ]
)


# Register all syntax classes in this module
for name in dir():
  syntax_registry.registerSyntaxClass(eval(name))

