"""
w2lapp.secinfo.py: Display (SSL) connection data

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: conninfo.py,v 1.58 2005/06/17 16:19:38 michael Exp $
"""

import time,ldap,w2lapp.cnf,pyweblib.sslenv,w2lapp.core,w2lapp.gui,msgzip, \
       w2lapp.schema.viewer

from w2lapp.core import utf2display

from w2lapp.session import session

##############################################################################
# Connection info
##############################################################################

def w2l_ConnInfo(sid,outf,command,form,ls,dn,env):

  protocol_version = ls.l.get_option(ldap.OPT_PROTOCOL_VERSION)

  ldap_managedsait = form.getInputValue('ldap_managedsait',[None])[0]
  if ldap_managedsait!=None:
    if protocol_version<ldap.VERSION3:
      raise w2lapp.core.ErrorExit(
        'Control ManageDsaIT (2.16.840.1.113730.3.4.2) not available with LDAPv%d.' % (protocol_version)
      )
    ls.manageDsaIt(int(ldap_managedsait))

  # Set the LDAP connection option for deferencing aliases
  if 'ldap_deref' in form.inputFieldNames:
    ls.l.deref = (int(form.field['ldap_deref'].value[0]))
  
  context_menu_list = []

  if protocol_version>=ldap.VERSION3:
    context_menu_list.append(
      form.applAnchor('read','Root DSE',sid,[('dn','')])
    )
  context_menu_list.extend(w2lapp.schema.viewer.SchemaContextMenu(sid,form,ls,dn))

  config_dn_list = []

  if ls.rootDSE.has_key('monitorContext'):
    context_menu_list.append(form.applAnchor('read','Monitor',sid,[('dn',ls.rootDSE['monitorContext'][0])]))
  else:
    config_dn_list.append(('CN=MONITOR','Monitor'))

  if ls.rootDSE.has_key('configContext'):
    # OpenLDAP 2.3+
    context_menu_list.append(form.applAnchor('read','Config',sid,[('dn',ls.rootDSE['configContext'][0])]))
  elif ls.rootDSE.has_key('configurationNamingContext'):
    # MS AD
    context_menu_list.append(form.applAnchor('read','AD Configuration',sid,[('dn',ls.rootDSE['configurationNamingContext'][0])]))
  else:
    config_dn_list.extend([
      ('CN=CONFIG','Config'),
      ('CN=Configuration','Configuration'),
      ('cn=ldbm','Database'),
    ])

  for config_dn,txt in config_dn_list:
    try:
      entry_exists = ls.existingEntry(config_dn,suppress_referrals=1)
    except ldap.LDAPError:
      pass
    else:
      if entry_exists:
        context_menu_list.append(form.applAnchor('read',txt,sid,[('dn',config_dn)]))

  if ls.rootDSE.has_key('schemaNamingContext'):
    # MS AD schema configuration
    context_menu_list.append(form.applAnchor('read','AD Schema Configuration',sid,[('dn',ls.rootDSE['schemaNamingContext'][0])]))

  w2lapp.gui.TopSection(
    sid,outf,form,ls,dn,
    'Connection Info',
    w2lapp.gui.MainMenu(sid,form,ls,dn),
    context_menu_list=context_menu_list
  )

  uptime = (time.time()-w2lapp.core.startUpTime)/60

  outf.write("""
    <div id="Message" class="Main">
    <h1>Connection Info</h1>
    <p>%s</p>
    <h2>Gateway statistics</h2>
    <table summary="Gateway statistics">
    <tr>
      <td>Uptime:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>LDAP connections:</td>
      <td>%d</td>
    </tr>
    </table>
    """ % (
      form.formHTML('read','Go to entry',sid,'GET',[],extrastr=form.field['dn'].inputHTML()),
      '%02d:%02d' % (int(uptime/60),uptime%60),
      int(len(session.sessiondict)/2),
    )
  )

  if protocol_version>=ldap.VERSION3:
    manageDsaItInfo = '%s | %s' % (
      {0:'disabled',1:'enabled'}[ls.manageDsaItenabled],
      form.applAnchor(
        'conninfo',
        {1:'Disable',0:'Enable'}[ls.manageDsaItenabled],
        sid,
        [('dn',dn),('ldap_managedsait',str(int(not ls.manageDsaItenabled)))]
      )
    )
  else:
    manageDsaItInfo = 'not available'

  derefAliasesStr = str(ls.l.get_option(ldap.OPT_DEREF))
  derefAliasesMenuList = []
  for option_value,option_text in form.field['ldap_deref'].options:
    if option_value!=derefAliasesStr:
      derefAliasesMenuList.append(
        form.applAnchor(
          'conninfo',option_text.encode(form.accept_charset),sid,
          [('dn',dn),('ldap_deref',option_value)]
        )
      )

  outf.write("""
    <h2>LDAP connection</h2>
    <table summary="LDAP connection">
    <tr>
      <td>LDAPv%d connection to:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>Manage DSA IT:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>Alias dereferencing:</td>
      <td>%s (%s)<br>%s</td>
    </tr>
    <tr>
      <td>Server vendor info:</td>
      <td>%s<br>%s</td>
    </tr>
    <tr>
      <td>Character set/encoding:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>Cached items:</td>
      <td>%d</td>
    </tr>
    <tr>
      <td>Secured Connection:</td>
      <td>%s</td>
    </tr>
    <tr>
      <td>Bound as:</td>
      <td>
    """ % (
      protocol_version,
      ls.uri.encode('ascii'),
      manageDsaItInfo,
      {
        ldap.DEREF_NEVER:u'never',
        ldap.DEREF_SEARCHING:u'searching',
        ldap.DEREF_FINDING:u'finding',
        ldap.DEREF_ALWAYS:u'always'
      }[ls.l.get_option(ldap.OPT_DEREF)].encode(form.accept_charset),
      derefAliasesStr,
      ' | '.join(derefAliasesMenuList),
      utf2display(form.accept_charset,(ls.__dict__.get('vendorName',None) or ['unknown'])[0]),
      utf2display(form.accept_charset,(ls.__dict__.get('vendorVersion',None) or ['unknown'])[0]),
      ls.charset.upper(),
      len(ls.read_cache),
      {0:'no',1:'yes'}[ls.secureConn>0],
    )
  )
  if ls.who:
    outf.write(
      ' | '.join([
        w2lapp.gui.DisplayDN(sid,form,ls,ls.who,commandbutton=0),
        form.applAnchor('read','Read',sid,[('dn',ls.who)]),
        form.applAnchor('passwd','Password',sid,[('dn',ls.who),('passwd_who',ls.who)]),
      ])
    )
  else:
    outf.write('anonymous')
  outf.write('</td>\n</tr>\n</table>\n')

  if isinstance(outf,msgzip.GzipFile):
    compresslevel = outf.compresslevel
  else:
    compresslevel = None
  outf.write("""
    <h2>HTTP connection</h2>
    <table summary="HTTP connection">
      <tr><td>Your IP address:</td><td>%s</td></tr>
      <tr><td>Server signature:</td><td>%s</td></tr>
      <tr><td>Character set/encoding:</td><td>%s</td></tr>
      <tr><td>GZIP compression level:</td><td>%s</td></tr>
      <tr><td>Cross-check vars in use:</td><td>%s</td></tr>
    </table>
    <h2>SSL</h2>
    """ % (
      w2lapp.core.guessClientAddr(env),
      form.env.get('SERVER_SIGNATURE',''),
      form.accept_charset.upper(),
      str(compresslevel),
      ', '.join(session.sessiondict['__session_checkvars__'+sid].keys()),
    )
  )
  ssl_valid_dn = w2lapp.cnf.GetParam(ls,'ssl_valid_dn','')
  ssl_valid_idn = w2lapp.cnf.GetParam(ls,'ssl_valid_idn','')
  pyweblib.sslenv.PrintSecInfo(env,w2lapp.cnf.misc.sec_sslacceptedciphers,ssl_valid_dn,ssl_valid_idn,outf)
  outf.write('</div>')
  w2lapp.gui.PrintFooter(outf,form)
