"""
w2lapp.schema: Module package for application-specific
               (pseudo-)schema handling

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: __init__.py,v 1.23 2005/06/02 07:29:22 michael Exp $
"""


import sys,ldap,ldap.schema

ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES.update({
  '1.3.6.1.4.1.1466.115.121.1.23':None,
  # From draft-sermersheim-nds-ldap-schema
  '2.16.840.1.113719.1.1.5.1.12':None,
  '2.16.840.1.113719.1.1.5.1.13':None,
})

# Work around a bug in older versions of python-ldap which
# declared the BitString and OctetString syntaxes as not human-readable
try:
  del ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES['1.3.6.1.4.1.1466.115.121.1.6']
  del ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES['1.3.6.1.4.1.1466.115.121.1.40']
except KeyError:
  pass

def no_userapp_attr(schema,attr_type_name):
  """
  Returns 1 if the attribute type specified by the schema
  element instance attr_se is considered operational and therefore
  should not be modified by the user.
  """
  attr_type_se = schema.get_obj(ldap.schema.AttributeType,attr_type_name)
  if attr_type_se is None:
    return 0
  return attr_type_se.usage!=0 or attr_type_se.no_user_mod


def no_humanreadable_attr(schema,attr_type_name):
  """
  Returns 1 if the attribute type specified by the schema
  element instance attr_se cannot be displayed human readable form.
  """
  attr_type_se = schema.get_obj(ldap.schema.AttributeType,attr_type_name)
  if attr_type_se is None:
    return 0
  syntax_oid = attr_type_se.__dict__.get('syntax',None)
  if syntax_oid!=None:
    syntax_se = schema.get_obj(ldap.schema.LDAPSyntax,syntax_oid)
    if syntax_se!=None and syntax_se.not_human_readable:
      return 1
  return \
    ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES.has_key(syntax_oid) or \
    attr_type_name.endswith(';binary')


def object_class_categories(sub_schema,object_classes):
  """
  Split a list of object class identifiers (name or OID)
  into three lists of categories of object classes.
  """
  ObjectClass = ldap.schema.ObjectClass
  kind = [
    ldap.cidict.cidict(),
    ldap.cidict.cidict(),
    ldap.cidict.cidict()
  ]
  for nameoroid in object_classes:
    oc_obj = sub_schema.get_obj(ObjectClass,nameoroid)
    if oc_obj is None:
      continue
    kind[oc_obj.kind][nameoroid] = None
  for k in range(3):
    l = kind[k].keys()
    l.sort()
    kind[k] = l
  return tuple(kind)


def parse_fake_schema(ldap_def):
  for k in ldap_def.keys():
    if not hasattr(ldap_def[k],'schema_uri'):
      continue
    try:
      schema_dn,schema = ldap.schema.urlfetch(
        ldap_def[k].schema_uri
      )
    except (IOError,OSError,ldap.LDAPError),e:
      # FIX ME!!! This does not work for running as FastCGI server
      sys.stderr.write('Error retrieving schema from %s: %s\n' % (
        ldap_def[k].schema_uri,str(e)))
    else:
      if schema!=None:
        ldap_def[k]._schema = schema

